/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.frameworks.portletcontainer.portalframework;

import java.util.*;
import javax.portlet.PortletMode;
import javax.portlet.WindowState;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.xml.namespace.QName;
import org.apache.commons.lang.StringUtils;
import org.exoplatform.container.ExoContainer;
import org.exoplatform.services.portletcontainer.PortletContainerException;
import org.exoplatform.services.portletcontainer.PortletContainerService;
import org.exoplatform.services.portletcontainer.pci.ActionInput;
import org.exoplatform.services.portletcontainer.pci.ActionOutput;
import org.exoplatform.services.portletcontainer.pci.EventInput;
import org.exoplatform.services.portletcontainer.pci.EventOutput;
import org.exoplatform.services.portletcontainer.pci.FragmentInput;
import org.exoplatform.services.portletcontainer.pci.FragmentOutput;
import org.exoplatform.services.portletcontainer.pci.Output;
import org.exoplatform.services.portletcontainer.pci.PortletData;
import org.exoplatform.services.portletcontainer.pci.RenderInput;
import org.exoplatform.services.portletcontainer.pci.RenderOutput;
import org.exoplatform.services.portletcontainer.pci.ResourceInput;
import org.exoplatform.services.portletcontainer.pci.ResourceOutput;
import org.exoplatform.services.portletcontainer.pci.model.DisplayName;
import org.exoplatform.services.portletcontainer.pci.model.EventDefinition;
import org.exoplatform.services.portletcontainer.pci.model.PortletApp;
import org.exoplatform.services.portletcontainer.plugins.pc.Constants;
import org.exoplatform.services.portletcontainer.plugins.pc.PortletApplicationsHolder;
import org.exoplatform.services.portletcontainer.plugins.pc.event.EventImpl;

/**
 * Created by The eXo Platform SARL .
 *
 * @author <a href="mailto:lautarul@gmail.com">Roman Pedchenko</a>
 * @version $Id$
 */

public class PortalFramework {

  protected ExoContainer portalContainer = null;

  protected PortletContainerService service = null;
  protected PortletApplicationsHolder holder = null;

  protected Map allPortletMetaData = null;

  protected HashMap wins = null;
  protected HashMap eventDelivery = null;
  protected HashMap sharedParams = null;
  protected HashMap portalParams = null;
  protected HashMap portletParams = null;
  protected Map actionParams = null;
  protected Map fragmentParams = null;
  protected HashSet changes = null;

  protected String target = null;
  protected int action = -1;
  protected String cntType = null;
  protected List events = null;
  protected String redirect = null;

  public PortalFramework(ExoContainer cnt) {
    portalContainer = cnt;
  }

  public String getPortalName() {
    return portalContainer.getContext().getName();
  }

  public String getTarget() {
    return target;
  }

  public int getAction() {
    return action;
  }

  public String getRedirect() {
    return redirect;
  }

  public HashSet getChanges() {
    return changes;
  }

  public Set getPortletNames() {
    return allPortletMetaData.keySet();
  }

  public Iterator getPortletModes(String pan, String pn) {
    return service.getPortletModes(pan, pn, cntType).iterator();
  }

  public Iterator getWindowStates(String pan, String pn) {
    return service.getWindowStates(pan, pn, cntType).iterator();
  }

  public String getPortletDisplayName(String plt) {
    PortletData portlet = (PortletData) allPortletMetaData.get(plt);
    return ((DisplayName) portlet.getDisplayName().get(0)).getDisplayName();
  }

  public WindowID2 getWindowID(String plt) {
    return (WindowID2) wins.get(plt);
  }

  public HashMap getPortalParams() {
    return portalParams;
  }
  public HashMap getPortletParams() {
    return portletParams;
  }

  public void setParams(HttpServletRequest httpRequest, String cntType) {
    this.cntType = cntType;
    portalParams = new HashMap();
    portletParams = new HashMap();
    actionParams = null;
    fragmentParams = null;
    events = new ArrayList();
    redirect = null;
    changes = new HashSet();

    Helper.parseParams(httpRequest, portalParams, portletParams);
    target = (String) portalParams.get(org.exoplatform.Constants.COMPONENT_PARAMETER);

    action = Helper.getActionType((String) portalParams.get(org.exoplatform.Constants.TYPE_PARAMETER));
    if (target == null)
      action = Constants.renderInt;

    if (!portalParams.isEmpty() && portalParams.containsKey(org.exoplatform.Constants.WINDOW_STATE_PARAMETER)) {
      if (((String) portalParams.get(org.exoplatform.Constants.WINDOW_STATE_PARAMETER)).equals(WindowState.MINIMIZED)) {
        Set set = wins.keySet();
        Iterator it = set.iterator();
        while (it.hasNext()) {
          String key = (String) it.next();
          WindowID2 window = (WindowID2) wins.get(key);
          window.setWindowState(WindowState.MINIMIZED);
        }
      }
    }

    String[] pn = StringUtils.split(target, "/");
    if (target != null) {
      WindowID2 win = (WindowID2) wins.get(target);
      if (portalParams.get(org.exoplatform.Constants.PORTLET_MODE_PARAMETER) != null) {
        changes.add(target);
        win.setPortletMode(Helper.getPortletMode((String) portalParams.get(org.exoplatform.Constants.PORTLET_MODE_PARAMETER),
          holder.getPortletModes(pn[0], pn[1], cntType)));
      }
      if (portalParams.get(org.exoplatform.Constants.WINDOW_STATE_PARAMETER) != null) {
        changes.add(target);
        win.setWindowState(Helper.getWindowState((String) portalParams.get(org.exoplatform.Constants.WINDOW_STATE_PARAMETER),
          holder.getWindowStates(pn[0], pn[1], cntType)));
      }
    }
  }

  public void init(HttpSession httpSession) {
    service =
      (PortletContainerService) portalContainer.getComponentInstanceOfType(PortletContainerService.class);
    holder =
      (PortletApplicationsHolder) portalContainer.getComponentInstanceOfType(PortletApplicationsHolder.class);
    initMaps();
  }
  
  protected void initMaps() {
    allPortletMetaData = service.getAllPortletMetaData();
    buildEventDeliveryAndSharedParamsMaps(allPortletMetaData);
    if (wins == null) {
      createWins(allPortletMetaData);
    }
  }

  protected void createWins(Map allPortletMetaData) {
    wins = new HashMap();
    Set keys = allPortletMetaData.keySet();
    Iterator i = keys.iterator();
    while(i.hasNext()) {
      String key = (String) i.next();
      PortletData portlet = (PortletData) allPortletMetaData.get(key);
      String portletApp = key.replace('.' , '/');
      String[] ss = StringUtils.split(portletApp, "/");

      WindowID2 windowID = new WindowID2();
      windowID.setOwner(org.exoplatform.Constants.ANON_USER);
      windowID.setPortletApplicationName(ss[0]);
      windowID.setPortletName(ss[1]);
      windowID.setUniqueID(portletApp);
      windowID.setPersistenceId(ss[0] + "II" + ss[1]);
      windowID.setPortletMode(PortletMode.VIEW);
      windowID.setWindowState(WindowState.NORMAL);
      wins.put(portletApp, windowID);
    }
  }

  // !!!!!!!!!!!!!!!!!!!!! supported-publishing-event
  // List getSupportedPublishingEvent()
  protected void buildEventDeliveryAndSharedParamsMaps(Map allPortletMetaData) {
    eventDelivery = new HashMap();
    sharedParams = new HashMap();
    Iterator i = allPortletMetaData.keySet().iterator();
    while(i.hasNext()) {
      String pname = (String) i.next();
      PortletData portlet = (PortletData) allPortletMetaData.get(pname);
      // shared params
      List srp = portlet.getSupportedSharedRenderParameter();
      if (srp != null) {
        sharedParams.put(pname, srp);
      }
      // event delivery
      List spe = portlet.getSupportedProcessingEvent();
      if (spe == null)
        continue;
      Iterator i1 = spe.iterator();
      while (i1.hasNext()) {
        QName ename = new QName((String)i1.next());
        List dl = (List) eventDelivery.get(ename);
        if (dl == null) {
          dl = new ArrayList();
          eventDelivery.put(ename, dl);
        }
        dl.add(pname);
      }
    }
  }

  protected HashMap fillSharedParams(String pname, HashMap params) {
    HashMap result = new HashMap();
    List srp = (List) sharedParams.get(pname);
    if (srp != null) {
      Iterator i = params.keySet().iterator();
      while (i.hasNext()) {
        String rname = (String) i.next();
        if (srp.contains(rname)) {
          result.put(rname, params.get(rname));
        }
          
      }
    }
    return result;
  }

  protected boolean checkEventValueType(EventInfo event) {
    if (event.value.getValue() == null)
      return false;
    String[] hs = StringUtils.split(event.target, "/");
		PortletApp portletApp = holder.getPortletApplication(hs[0]);
		List eds = portletApp.getEventDefinition();
    for (Iterator i = eds.iterator(); i.hasNext();) {
      EventDefinition ed = (EventDefinition) i.next();
      // !!! poate sa face aici o verificare de numele alte de portlet decat preffered-ul
      if (!event.name.equals(ed.getPrefferedName()))
        continue;
      if (ed.getJavaClass() != null) {
        Class jc;
        try {
          jc = event.value.getValue().getClass().getClassLoader().loadClass(ed.getJavaClass());
        } catch (Exception e) { jc = null; }
        if (jc != null) {
          return jc.isInstance(event.value.getValue());
        } else
          return false;
      } else
        return true;
    }
    return false;
  }
  
  protected void addEvents(List events, Map newEvents) {
    for (Iterator i = newEvents.keySet().iterator(); i.hasNext();) {
      QName name = (QName) i.next();
      List pl = (List) eventDelivery.get(name);
      if (pl == null) {
        pl = new ArrayList();
        eventDelivery.put(name, pl);
      }
      Object value = (Object) newEvents.get(name);
      for (Iterator i1 = pl.iterator(); i1.hasNext();) {
        String pname = (String) i1.next();
        EventInfo event = new EventInfo();
        event.name = name;
        event.value = new EventImpl(name, value);
        event.target = pname;
        if (checkEventValueType(event))
          events.add(event);
      }
    }
  }

  // --- input objects creators ---
  
  public FragmentInput createFragmentInput(ServletContext ctx) {
    FragmentInput fragmentInput = new FragmentInput();
    WindowID2 win = (WindowID2) wins.get(target);
    fragmentInput.setWindowID(win);
    fragmentInput.setBaseURL("/" + ctx.getServletContextName() + "/?"
      + org.exoplatform.Constants.COMPONENT_PARAMETER + "=" + win.getUniqueID());
    fragmentInput.setUserAttributes(new HashMap());
    fragmentInput.setMarkup(cntType);
    fragmentInput.setRenderParameters(portletParams);
    fragmentInput.setPortletMode(win.getPortletMode());
    fragmentInput.setWindowState(win.getWindowState());
    return fragmentInput;
  }
 
  public ResourceInput createResourceInput(ServletContext ctx) {
    ResourceInput resourceInput = new ResourceInput();
    WindowID2 win = (WindowID2) wins.get(target);
    resourceInput.setWindowID(win);
    resourceInput.setBaseURL("/" + ctx.getServletContextName() + "/?"
      + org.exoplatform.Constants.COMPONENT_PARAMETER + "=" + win.getUniqueID());
    resourceInput.setUserAttributes(new HashMap());
    resourceInput.setMarkup(cntType);
    resourceInput.setRenderParameters(portletParams);
    resourceInput.setPortletMode(win.getPortletMode());
    resourceInput.setWindowState(win.getWindowState());
    resourceInput.setResourceID((String) portalParams.get(org.exoplatform.Constants.RESOURCE_ID_PARAMETER));
    return resourceInput;
  }

  public ActionInput createActionInput(ServletContext ctx) {
    ActionInput actionInput = new ActionInput();
    WindowID2 win = (WindowID2) wins.get(target);
    actionInput.setWindowID(win);
    actionInput.setBaseURL("/" + ctx.getServletContextName() + "/?"
      + org.exoplatform.Constants.COMPONENT_PARAMETER + "=" + win.getUniqueID());
    actionInput.setUserAttributes(new HashMap());
    actionInput.setMarkup(cntType);
    actionInput.setPortletMode(win.getPortletMode());
    actionInput.setWindowState(win.getWindowState());
    actionInput.setStateChangeAuthorized(true);
    return actionInput;
  }
 
  public EventInput createEventInput(ServletContext ctx, EventInfo event) {
    changes.add(event.target);
    EventInput eventInput = new EventInput();
    WindowID2 win = (WindowID2) wins.get(event.target);
    eventInput.setWindowID(win);
    eventInput.setBaseURL("/" + ctx.getServletContextName() + "/?"
      + org.exoplatform.Constants.COMPONENT_PARAMETER + "=" + win.getUniqueID());
    eventInput.setUserAttributes(new HashMap());
    eventInput.setMarkup(cntType);
    eventInput.setPortletMode(win.getPortletMode());
    eventInput.setWindowState(win.getWindowState());
    eventInput.setEvent(event.value);
    return eventInput;
  }
 
  public RenderInput createRenderInput(ServletContext ctx, String plt) {
    RenderInput renderInput = new RenderInput();
    WindowID2 win = (WindowID2) wins.get(plt);
    renderInput.setWindowID(win);
    renderInput.setBaseURL("/" + ctx.getServletContextName() + "/?"
      + org.exoplatform.Constants.COMPONENT_PARAMETER + "=" + win.getUniqueID());
    renderInput.setUserAttributes(new HashMap());
    renderInput.setMarkup(cntType);
    if (target != null && target.equals(plt) && action != Constants.actionInt) {
      renderInput.setRenderParameters(portletParams);
    } else {
      renderInput.setRenderParameters(fillSharedParams(plt, portletParams));
    }
    if (actionParams != null)
      renderInput.getRenderParameters().putAll(actionParams);
    if (fragmentParams != null)
      renderInput.getRenderParameters().putAll(fragmentParams);
    renderInput.setPortletMode(win.getPortletMode());
    renderInput.setWindowState(win.getWindowState());
    return renderInput;
  }

  // --- portlet method callers ---

  public FragmentOutput serveFragment(HttpServletRequest httpRequest, HttpServletResponse httpResponse,
    FragmentInput fragmentInput) throws PortletContainerException {
    FragmentOutput o = service.serveFragment(httpRequest, httpResponse, fragmentInput);
    fragmentParams = o.getRenderParameters();
    addEvents(events, o.getEvents());
    redirect = (String) o.getProperties().get(Output.SEND_REDIRECT);
    WindowID2 win = (WindowID2) wins.get(target);
    if (o.getNextMode() != null)
      win.setPortletMode(o.getNextMode());
    if (o.getNextState() != null)
      win.setWindowState(o.getNextState());
    return o;
  }

  public ResourceOutput serveResource(HttpServletRequest httpRequest, HttpServletResponse httpResponse,
    ResourceInput resourceInput) throws PortletContainerException {
    ResourceOutput o = service.serveResource(httpRequest, httpResponse, resourceInput);
    return o;
  }

  public ActionOutput processAction(HttpServletRequest httpRequest, HttpServletResponse httpResponse,
    ActionInput actionInput) throws PortletContainerException {
    ActionOutput o = service.processAction(httpRequest, httpResponse, actionInput);
    actionParams = o.getRenderParameters();
    addEvents(events, o.getEvents());
    redirect = (String) o.getProperties().get(Output.SEND_REDIRECT);
    WindowID2 win = (WindowID2) wins.get(target);
    if (o.getNextMode() != null)
      win.setPortletMode(o.getNextMode());
    if (o.getNextState() != null)
      win.setWindowState(o.getNextState());
    return o;
  }

  public EventOutput processEvent(HttpServletRequest httpRequest, HttpServletResponse httpResponse,
    EventInput eventInput, EventInfo event) throws PortletContainerException {
    EventOutput o = service.processEvent(httpRequest, httpResponse, eventInput);
    addEvents(events, o.getEvents());
    WindowID2 win = (WindowID2) wins.get(event.target);
    if (o.getNextMode() != null)
      win.setPortletMode(o.getNextMode());
    if (o.getNextState() != null)
      win.setWindowState(o.getNextState());
    return o;
  }

  public RenderOutput render(HttpServletRequest httpRequest, HttpServletResponse httpResponse,
    RenderInput renderInput) throws PortletContainerException {
    RenderOutput o = service.render(httpRequest, httpResponse, renderInput);
    return o;
  }

  // --- ---

  public void dispatchEvents(ServletContext ctx, HttpServletRequest httpRequest, HttpServletResponse httpResponse)
    throws PortletContainerException {
    while (events.size() > 0) {
      EventInfo event = (EventInfo) events.get(0);
      events.remove(0);
      EventInput eventInput = createEventInput(ctx, event);
      EventOutput o = processEvent(httpRequest, httpResponse, eventInput, event);
    }
  }

}
