package org.exoplatform.services.portletcontainer.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Set;
import javax.portlet.PortletMode;
import javax.portlet.WindowState;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.exoplatform.services.portletcontainer.PortletContainerException;
import org.exoplatform.services.portletcontainer.PortletContainerPlugin;
import org.exoplatform.services.portletcontainer.PortletContainerService;
import org.exoplatform.services.portletcontainer.pci.*;
import org.picocontainer.Startable;

/**
 * Created by the Exo Development team. Author : Mestrallet Benjamin
 * benjamin.mestrallet@exoplatform.com
 */
public class PortletContainerServiceImpl implements PortletContainerService,  Startable {

//  private PortletContainerDispatcher dispatcher_;
  private ArrayList plugins;

  public PortletContainerServiceImpl(/* PortletContainerDispatcher dispatcher*/) {
//    dispatcher_ = dispatcher ;
    plugins = new ArrayList();
  }

  public void addPlugin(PortletContainerPlugin plugin) {
System.out.println(" PortletContainerServiceImpl plugin registered: " + plugin);
    plugins.add(plugin);
  }
  
  public void start() {
  }

  public void stop() {
  }

  public void setPortletContainerName(String containerName) {
//    dispatcher_.setPortletContainerName(containerName);
  }

  public void setMajorVersion(int majorVersion) {
//    dispatcher_.setMajorVersion(majorVersion);
  }

  public void setMinorVersion(int minorVersion) {
//    dispatcher_.setMinorVersion(minorVersion);
  }

  public void setProperties(Map properties) {
//    dispatcher_.setProperties(properties);
  }

  public void setSupportedPortletModesWithDescriptions(Collection portletModes) {
//    dispatcher_.setSupportedPortletModesWithDescriptions(portletModes);
  }

  public void setSupportedWindowStatesWithDescriptions(Collection windowStates) {
//    dispatcher_.setSupportedWindowStatesWithDescriptions(windowStates);
  }

  public Collection getSupportedPortletModes() {
    ArrayList res = new ArrayList();
    for (Object plugin : plugins.toArray()) {
      res.addAll(((PortletContainerPlugin)plugin).getSupportedPortletModes());
    }
    return res;
  }

  public Collection getSupportedWindowStates() {
    ArrayList res = new ArrayList();
    for (Object plugin : plugins.toArray()) {
      res.addAll(((PortletContainerPlugin)plugin).getSupportedWindowStates());
    }
    return res;
  }

  public Collection getSupportedPortletModesWithDescriptions() {
    ArrayList res = new ArrayList();
    for (Object plugin : plugins.toArray()) {
      res.addAll(((PortletContainerPlugin)plugin).getSupportedPortletModesWithDescriptions());
    }
    return res;
  }

  public Collection getSupportedWindowStatesWithDescriptions() {
    ArrayList res = new ArrayList();
    for (Object plugin : plugins.toArray()) {
      res.addAll(((PortletContainerPlugin)plugin).getSupportedWindowStatesWithDescriptions());
    }
    return res;
  }

  public Collection getPortletModes(String portletAppName, String portletName,
      String markup) {
    ArrayList res = new ArrayList();
    for (Object plugin : plugins.toArray()) {
      res.addAll(((PortletContainerPlugin)plugin).getPortletModes(portletAppName, portletName, markup));
    }
    return res;
  }

  public boolean isModeSuported(String portletAppName, String portletName,
      String markup, PortletMode mode) {
    boolean res = false;
    for (Object plugin : plugins.toArray()) {
      res = res || ((PortletContainerPlugin)plugin).isModeSuported(portletAppName, portletName, markup, mode);
    }
    return res;
  }

  public Collection getWindowStates(String portletAppName, String portletName,
      String markup) {
    ArrayList res = new ArrayList();
    for (Object plugin : plugins.toArray()) {
      res.addAll(((PortletContainerPlugin)plugin).getWindowStates(portletAppName, portletName, markup));
    }
    return res;
  }

  public boolean isStateSupported(String portletAppName, String portletName,
      String markup, WindowState state) {
    boolean res = false;
    for (Object plugin : plugins.toArray()) {
      res = res || ((PortletContainerPlugin)plugin).isStateSupported(portletAppName, portletName, markup, state);
    }
    return res;
  }

  public Map getAllPortletMetaData() {
    HashMap res = new HashMap();
    for (Object plugin : plugins.toArray()) {
      res.putAll(((PortletContainerPlugin)plugin).getAllPortletMetaData());
    }
    return res;
  }

  public ResourceBundle getBundle(HttpServletRequest request,
      HttpServletResponse response, String portletAppName, String portletName,
      Locale locale) throws PortletContainerException {
    ResourceBundle res = null;
    for (Object plugin : plugins.toArray()) {
      res = ((PortletContainerPlugin)plugin).getBundle(request, response, portletAppName, portletName, locale);
      if (res != null)
        break;
    }
    return res;
  }

  protected PortletContainerPlugin findPluginByPAPPName(String papp) {
    for (Object plugin : plugins.toArray()) {
      Map plts = ((PortletContainerPlugin)plugin).getAllPortletMetaData();
      Set keys = plts.keySet();
      Iterator i = keys.iterator();
      while(i.hasNext()) {
        String s = (String) i.next();
        if (s.startsWith(papp + "/"))
          return (PortletContainerPlugin)plugin;
      }
    }
    return null;
  }
  
  public void setPortletPreference(Input input, Map preferences)
      throws PortletContainerException {
    PortletContainerPlugin plugin = findPluginByPAPPName(input.getWindowID().getPortletApplicationName());
    ((PortletContainerPlugin)plugin).setPortletPreference(input, preferences);
  }

  public Map getPortletPreference(Input input) {
    HashMap res = new HashMap();
    for (Object plugin : plugins.toArray()) {
      res.putAll(((PortletContainerPlugin)plugin).getPortletPreference(input));
    }
    return res;
  }

  public ActionOutput processAction(HttpServletRequest request,
      HttpServletResponse response, ActionInput input)
      throws PortletContainerException {
    return findPluginByPAPPName(input.getWindowID().getPortletApplicationName()).processAction(request, response, input);
  }


  public EventOutput processEvent(HttpServletRequest request,
      HttpServletResponse response, EventInput input)
      throws PortletContainerException {
    return findPluginByPAPPName(input.getWindowID().getPortletApplicationName()).processEvent(request, response, input);
  }


  public ResourceOutput serveResource(HttpServletRequest request,
      HttpServletResponse response, ResourceInput input)
      throws PortletContainerException {
    return findPluginByPAPPName(input.getWindowID().getPortletApplicationName()).serveResource(request, response, input);
  }

  public FragmentOutput serveFragment(HttpServletRequest request,
      HttpServletResponse response, FragmentInput input)
      throws PortletContainerException {
    return findPluginByPAPPName(input.getWindowID().getPortletApplicationName()).serveFragment(request, response, input);
  }
  
  public RenderOutput render(HttpServletRequest request,
      HttpServletResponse response, RenderInput input)
      throws PortletContainerException {
    return findPluginByPAPPName(input.getWindowID().getPortletApplicationName()).render(request, response, input);
  }

  // sessions replication
  public void sendAttrs(HttpServletRequest request, HttpServletResponse response,
      Map attrs, String portletApplicationName) throws PortletContainerException{
    findPluginByPAPPName(portletApplicationName).sendAttrs(request,response,attrs,portletApplicationName);
      }
  
}
